//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 316227767
#define m_grid 1
#define n_grid 68
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_isamax[] = { 
10, 4, 4, 4, 35, 35, 35, 36, 22, 13, 8, 9, 14, 4, 3, 3, 16, 2, 2, 1, 2, 5, 3, 3, 6, 6, 7, 11, 23, 11, 11, 23, 17, 11, 12, 12, 18, 13, 27, 28, 32, 19, 30, 29, 31, 33, 25, 26, 34, 19, 21, 14, 14, 20, 15, 15, 14, 14, 15, 15, 15, 15, 15, 15, 15, 15, 15, 24, 
};

int atune_isamax(isamax_arguments)
{
   int bestkernel = besttable_isamax[n_idx];
   switch (bestkernel)
   {
   case 1:
      isamax_v1<1, 16, 1, 1>(x_d, y_d, n);
      break;
   case 2:
      isamax_v1<1, 16, 1, 2>(x_d, y_d, n);
      break;
   case 3:
      isamax_v1<1, 16, 1, 4>(x_d, y_d, n);
      break;
   case 4:
      isamax_v1<1, 32, 1, 1>(x_d, y_d, n);
      break;
   case 5:
      isamax_v1<1, 32, 1, 2>(x_d, y_d, n);
      break;
   case 6:
      isamax_v1<1, 32, 1, 4>(x_d, y_d, n);
      break;
   case 7:
      isamax_v1<1, 32, 1, 8>(x_d, y_d, n);
      break;
   case 8:
      isamax_v1<1, 32, 1, 16>(x_d, y_d, n);
      break;
   case 9:
      isamax_v1<1, 32, 1, 32>(x_d, y_d, n);
      break;
   case 10:
      isamax_v1<1, 32, 1, 64>(x_d, y_d, n);
      break;
   case 11:
      isamax_v1<1, 64, 1, 4>(x_d, y_d, n);
      break;
   case 12:
      isamax_v1<1, 64, 1, 8>(x_d, y_d, n);
      break;
   case 13:
      isamax_v1<1, 64, 1, 16>(x_d, y_d, n);
      break;
   case 14:
      isamax_v1<1, 64, 1, 32>(x_d, y_d, n);
      break;
   case 15:
      isamax_v1<1, 64, 1, 64>(x_d, y_d, n);
      break;
   case 16:
      isamax_v1<1, 64, 1, 256>(x_d, y_d, n);
      break;
   case 17:
      isamax_v1<1, 96, 1, 4>(x_d, y_d, n);
      break;
   case 18:
      isamax_v1<1, 96, 1, 8>(x_d, y_d, n);
      break;
   case 19:
      isamax_v1<1, 96, 1, 16>(x_d, y_d, n);
      break;
   case 20:
      isamax_v1<1, 96, 1, 32>(x_d, y_d, n);
      break;
   case 21:
      isamax_v1<1, 96, 1, 64>(x_d, y_d, n);
      break;
   case 22:
      isamax_v1<1, 96, 1, 256>(x_d, y_d, n);
      break;
   case 23:
      isamax_v1<1, 128, 1, 2>(x_d, y_d, n);
      break;
   case 24:
      isamax_v1<1, 160, 1, 32>(x_d, y_d, n);
      break;
   case 25:
      isamax_v1<1, 160, 1, 64>(x_d, y_d, n);
      break;
   case 26:
      isamax_v1<1, 160, 1, 128>(x_d, y_d, n);
      break;
   case 27:
      isamax_v1<1, 192, 1, 8>(x_d, y_d, n);
      break;
   case 28:
      isamax_v1<1, 256, 1, 16>(x_d, y_d, n);
      break;
   case 29:
      isamax_v1<1, 288, 1, 16>(x_d, y_d, n);
      break;
   case 30:
      isamax_v1<1, 320, 1, 16>(x_d, y_d, n);
      break;
   case 31:
      isamax_v1<1, 320, 1, 32>(x_d, y_d, n);
      break;
   case 32:
      isamax_v1<1, 352, 1, 16>(x_d, y_d, n);
      break;
   case 33:
      isamax_v1<1, 352, 1, 32>(x_d, y_d, n);
      break;
   case 34:
      isamax_v1<1, 448, 1, 64>(x_d, y_d, n);
      break;
   case 35:
      isamax_v1<1, 480, 1, 16>(x_d, y_d, n);
      break;
   case 36:
      isamax_v1<1, 512, 1, 8>(x_d, y_d, n);
      break;
   default:
      printf("\nisamax.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
