//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 316227767
#define m_grid 1
#define n_grid 272
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_samax[] = { 
18, 18, 13, 18, 26, 27, 10, 13, 10, 44, 5, 2, 17, 22, 16, 2, 9, 11, 15, 11, 7, 9, 7, 20, 7, 8, 7, 11, 4, 6, 16, 7, 14, 3, 29, 5, 4, 5, 1, 9, 9, 28, 28, 16, 22, 14, 11, 21, 9, 18, 22, 22, 22, 31, 19, 25, 22, 25, 30, 5, 28, 28, 30, 28, 28, 29, 29, 41, 11, 40, 17, 40, 11, 45, 45, 11, 45, 29, 11, 11, 11, 25, 11, 11, 11, 32, 11, 28, 45, 23, 42, 25, 25, 47, 25, 25, 26, 27, 25, 40, 40, 41, 26, 40, 25, 25, 41, 54, 40, 62, 40, 62, 62, 66, 41, 12, 62, 66, 50, 57, 57, 63, 41, 41, 66, 18, 27, 63, 63, 28, 63, 64, 63, 42, 13, 68, 28, 63, 67, 66, 13, 58, 43, 43, 33, 41, 53, 64, 48, 42, 49, 49, 56, 38, 29, 29, 43, 58, 55, 28, 53, 29, 51, 39, 43, 43, 59, 56, 65, 29, 69, 53, 56, 59, 59, 53, 53, 60, 37, 44, 70, 30, 44, 44, 60, 70, 70, 61, 30, 52, 29, 35, 31, 35, 28, 29, 28, 29, 28, 34, 34, 28, 35, 24, 28, 28, 28, 35, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 28, 34, 34, 34, 36, 36, 43, 43, 43, 43, 46, 46, 69, 
};

int atune_samax(samax_arguments)
{
   int bestkernel = besttable_samax[n_idx];
   switch (bestkernel)
   {
   case 1:
      samax_v1<1, 16, 1, 1>(x_d, y_d, n);
      break;
   case 2:
      samax_v1<1, 16, 1, 2>(x_d, y_d, n);
      break;
   case 3:
      samax_v1<1, 16, 1, 4>(x_d, y_d, n);
      break;
   case 4:
      samax_v1<1, 16, 1, 8>(x_d, y_d, n);
      break;
   case 5:
      samax_v1<1, 16, 1, 16>(x_d, y_d, n);
      break;
   case 6:
      samax_v1<1, 16, 1, 32>(x_d, y_d, n);
      break;
   case 7:
      samax_v1<1, 16, 1, 64>(x_d, y_d, n);
      break;
   case 8:
      samax_v1<1, 16, 1, 128>(x_d, y_d, n);
      break;
   case 9:
      samax_v1<1, 16, 1, 256>(x_d, y_d, n);
      break;
   case 10:
      samax_v1<1, 32, 1, 1>(x_d, y_d, n);
      break;
   case 11:
      samax_v1<1, 32, 1, 2>(x_d, y_d, n);
      break;
   case 12:
      samax_v1<1, 32, 1, 8>(x_d, y_d, n);
      break;
   case 13:
      samax_v1<1, 32, 1, 16>(x_d, y_d, n);
      break;
   case 14:
      samax_v1<1, 32, 1, 32>(x_d, y_d, n);
      break;
   case 15:
      samax_v1<1, 32, 1, 64>(x_d, y_d, n);
      break;
   case 16:
      samax_v1<1, 32, 1, 256>(x_d, y_d, n);
      break;
   case 17:
      samax_v1<1, 64, 1, 1>(x_d, y_d, n);
      break;
   case 18:
      samax_v1<1, 64, 1, 8>(x_d, y_d, n);
      break;
   case 19:
      samax_v1<1, 64, 1, 32>(x_d, y_d, n);
      break;
   case 20:
      samax_v1<1, 64, 1, 64>(x_d, y_d, n);
      break;
   case 21:
      samax_v1<1, 64, 1, 128>(x_d, y_d, n);
      break;
   case 22:
      samax_v1<1, 64, 1, 256>(x_d, y_d, n);
      break;
   case 23:
      samax_v1<1, 96, 1, 2>(x_d, y_d, n);
      break;
   case 24:
      samax_v1<1, 96, 1, 16>(x_d, y_d, n);
      break;
   case 25:
      samax_v1<1, 128, 1, 2>(x_d, y_d, n);
      break;
   case 26:
      samax_v1<1, 128, 1, 4>(x_d, y_d, n);
      break;
   case 27:
      samax_v1<1, 128, 1, 8>(x_d, y_d, n);
      break;
   case 28:
      samax_v1<1, 128, 1, 16>(x_d, y_d, n);
      break;
   case 29:
      samax_v1<1, 128, 1, 32>(x_d, y_d, n);
      break;
   case 30:
      samax_v1<1, 128, 1, 64>(x_d, y_d, n);
      break;
   case 31:
      samax_v1<1, 128, 1, 256>(x_d, y_d, n);
      break;
   case 32:
      samax_v1<1, 160, 1, 2>(x_d, y_d, n);
      break;
   case 33:
      samax_v1<1, 160, 1, 8>(x_d, y_d, n);
      break;
   case 34:
      samax_v1<1, 160, 1, 16>(x_d, y_d, n);
      break;
   case 35:
      samax_v1<1, 192, 1, 8>(x_d, y_d, n);
      break;
   case 36:
      samax_v1<1, 192, 1, 16>(x_d, y_d, n);
      break;
   case 37:
      samax_v1<1, 192, 1, 32>(x_d, y_d, n);
      break;
   case 38:
      samax_v1<1, 224, 1, 8>(x_d, y_d, n);
      break;
   case 39:
      samax_v1<1, 224, 1, 16>(x_d, y_d, n);
      break;
   case 40:
      samax_v1<1, 256, 1, 2>(x_d, y_d, n);
      break;
   case 41:
      samax_v1<1, 256, 1, 4>(x_d, y_d, n);
      break;
   case 42:
      samax_v1<1, 256, 1, 8>(x_d, y_d, n);
      break;
   case 43:
      samax_v1<1, 256, 1, 16>(x_d, y_d, n);
      break;
   case 44:
      samax_v1<1, 256, 1, 32>(x_d, y_d, n);
      break;
   case 45:
      samax_v1<1, 256, 1, 128>(x_d, y_d, n);
      break;
   case 46:
      samax_v1<1, 288, 1, 16>(x_d, y_d, n);
      break;
   case 47:
      samax_v1<1, 320, 1, 2>(x_d, y_d, n);
      break;
   case 48:
      samax_v1<1, 320, 1, 4>(x_d, y_d, n);
      break;
   case 49:
      samax_v1<1, 320, 1, 8>(x_d, y_d, n);
      break;
   case 50:
      samax_v1<1, 352, 1, 2>(x_d, y_d, n);
      break;
   case 51:
      samax_v1<1, 352, 1, 16>(x_d, y_d, n);
      break;
   case 52:
      samax_v1<1, 352, 1, 64>(x_d, y_d, n);
      break;
   case 53:
      samax_v1<1, 384, 1, 16>(x_d, y_d, n);
      break;
   case 54:
      samax_v1<1, 416, 1, 1>(x_d, y_d, n);
      break;
   case 55:
      samax_v1<1, 416, 1, 8>(x_d, y_d, n);
      break;
   case 56:
      samax_v1<1, 416, 1, 16>(x_d, y_d, n);
      break;
   case 57:
      samax_v1<1, 448, 1, 4>(x_d, y_d, n);
      break;
   case 58:
      samax_v1<1, 448, 1, 8>(x_d, y_d, n);
      break;
   case 59:
      samax_v1<1, 448, 1, 16>(x_d, y_d, n);
      break;
   case 60:
      samax_v1<1, 448, 1, 32>(x_d, y_d, n);
      break;
   case 61:
      samax_v1<1, 448, 1, 64>(x_d, y_d, n);
      break;
   case 62:
      samax_v1<1, 480, 1, 1>(x_d, y_d, n);
      break;
   case 63:
      samax_v1<1, 480, 1, 4>(x_d, y_d, n);
      break;
   case 64:
      samax_v1<1, 480, 1, 8>(x_d, y_d, n);
      break;
   case 65:
      samax_v1<1, 480, 1, 16>(x_d, y_d, n);
      break;
   case 66:
      samax_v1<1, 512, 1, 2>(x_d, y_d, n);
      break;
   case 67:
      samax_v1<1, 512, 1, 4>(x_d, y_d, n);
      break;
   case 68:
      samax_v1<1, 512, 1, 8>(x_d, y_d, n);
      break;
   case 69:
      samax_v1<1, 512, 1, 16>(x_d, y_d, n);
      break;
   case 70:
      samax_v1<1, 512, 1, 32>(x_d, y_d, n);
      break;
   default:
      printf("\nsamax.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
