//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 316227767
#define m_grid 1
#define n_grid 272
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_sasum[] = { 
11, 15, 15, 11, 15, 15, 15, 11, 11, 11, 11, 11, 11, 11, 12, 11, 2, 3, 8, 6, 6, 12, 7, 3, 15, 6, 13, 8, 5, 28, 12, 8, 11, 5, 11, 10, 6, 11, 4, 5, 14, 3, 5, 3, 7, 6, 20, 1, 2, 23, 28, 5, 5, 18, 17, 18, 17, 13, 6, 3, 3, 1, 3, 1, 9, 9, 9, 3, 4, 3, 4, 9, 9, 28, 9, 9, 1, 4, 4, 1, 3, 3, 3, 4, 2, 28, 3, 3, 30, 9, 3, 3, 3, 3, 2, 28, 3, 19, 3, 3, 16, 3, 3, 3, 3, 3, 3, 28, 3, 30, 11, 28, 11, 17, 39, 11, 11, 30, 17, 29, 17, 41, 17, 41, 30, 42, 40, 24, 30, 40, 17, 41, 18, 46, 41, 41, 41, 41, 30, 30, 41, 25, 41, 41, 41, 41, 35, 41, 41, 33, 37, 33, 37, 37, 42, 38, 37, 31, 26, 31, 34, 34, 38, 38, 43, 27, 38, 44, 34, 45, 38, 34, 36, 34, 38, 43, 43, 32, 32, 20, 22, 22, 22, 22, 20, 20, 21, 21, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 21, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 21, 21, 21, 21, 21, 21, 21, 21, 21, 44, 44, 45, 
};

int atune_sasum(sasum_arguments)
{
   int bestkernel = besttable_sasum[n_idx];
   switch (bestkernel)
   {
   case 1:
      sasum_v1<1, 16, 1, 1>(x_d, y_d, n);
      break;
   case 2:
      sasum_v1<1, 16, 1, 2>(x_d, y_d, n);
      break;
   case 3:
      sasum_v1<1, 16, 1, 4>(x_d, y_d, n);
      break;
   case 4:
      sasum_v1<1, 16, 1, 8>(x_d, y_d, n);
      break;
   case 5:
      sasum_v1<1, 16, 1, 32>(x_d, y_d, n);
      break;
   case 6:
      sasum_v1<1, 16, 1, 64>(x_d, y_d, n);
      break;
   case 7:
      sasum_v1<1, 16, 1, 128>(x_d, y_d, n);
      break;
   case 8:
      sasum_v1<1, 16, 1, 256>(x_d, y_d, n);
      break;
   case 9:
      sasum_v1<1, 32, 1, 1>(x_d, y_d, n);
      break;
   case 10:
      sasum_v1<1, 32, 1, 2>(x_d, y_d, n);
      break;
   case 11:
      sasum_v1<1, 32, 1, 4>(x_d, y_d, n);
      break;
   case 12:
      sasum_v1<1, 32, 1, 8>(x_d, y_d, n);
      break;
   case 13:
      sasum_v1<1, 32, 1, 64>(x_d, y_d, n);
      break;
   case 14:
      sasum_v1<1, 32, 1, 128>(x_d, y_d, n);
      break;
   case 15:
      sasum_v1<1, 32, 1, 256>(x_d, y_d, n);
      break;
   case 16:
      sasum_v1<1, 64, 1, 1>(x_d, y_d, n);
      break;
   case 17:
      sasum_v1<1, 64, 1, 4>(x_d, y_d, n);
      break;
   case 18:
      sasum_v1<1, 64, 1, 8>(x_d, y_d, n);
      break;
   case 19:
      sasum_v1<1, 64, 1, 16>(x_d, y_d, n);
      break;
   case 20:
      sasum_v1<1, 64, 1, 32>(x_d, y_d, n);
      break;
   case 21:
      sasum_v1<1, 64, 1, 64>(x_d, y_d, n);
      break;
   case 22:
      sasum_v1<1, 64, 1, 128>(x_d, y_d, n);
      break;
   case 23:
      sasum_v1<1, 64, 1, 256>(x_d, y_d, n);
      break;
   case 24:
      sasum_v1<1, 96, 1, 2>(x_d, y_d, n);
      break;
   case 25:
      sasum_v1<1, 96, 1, 8>(x_d, y_d, n);
      break;
   case 26:
      sasum_v1<1, 96, 1, 16>(x_d, y_d, n);
      break;
   case 27:
      sasum_v1<1, 96, 1, 32>(x_d, y_d, n);
      break;
   case 28:
      sasum_v1<1, 128, 1, 1>(x_d, y_d, n);
      break;
   case 29:
      sasum_v1<1, 128, 1, 2>(x_d, y_d, n);
      break;
   case 30:
      sasum_v1<1, 128, 1, 4>(x_d, y_d, n);
      break;
   case 31:
      sasum_v1<1, 128, 1, 16>(x_d, y_d, n);
      break;
   case 32:
      sasum_v1<1, 128, 1, 32>(x_d, y_d, n);
      break;
   case 33:
      sasum_v1<1, 160, 1, 8>(x_d, y_d, n);
      break;
   case 34:
      sasum_v1<1, 160, 1, 16>(x_d, y_d, n);
      break;
   case 35:
      sasum_v1<1, 192, 1, 4>(x_d, y_d, n);
      break;
   case 36:
      sasum_v1<1, 192, 1, 16>(x_d, y_d, n);
      break;
   case 37:
      sasum_v1<1, 224, 1, 8>(x_d, y_d, n);
      break;
   case 38:
      sasum_v1<1, 224, 1, 16>(x_d, y_d, n);
      break;
   case 39:
      sasum_v1<1, 256, 1, 1>(x_d, y_d, n);
      break;
   case 40:
      sasum_v1<1, 256, 1, 2>(x_d, y_d, n);
      break;
   case 41:
      sasum_v1<1, 256, 1, 4>(x_d, y_d, n);
      break;
   case 42:
      sasum_v1<1, 256, 1, 8>(x_d, y_d, n);
      break;
   case 43:
      sasum_v1<1, 256, 1, 16>(x_d, y_d, n);
      break;
   case 44:
      sasum_v1<1, 416, 1, 16>(x_d, y_d, n);
      break;
   case 45:
      sasum_v1<1, 480, 1, 16>(x_d, y_d, n);
      break;
   case 46:
      sasum_v1<1, 512, 1, 2>(x_d, y_d, n);
      break;
   default:
      printf("\nsasum.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
