//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 316227767
#define m_grid 1
#define n_grid 272
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_saxpy[] = { 
3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 24, 4, 6, 4, 4, 7, 4, 2, 4, 4, 25, 10, 38, 10, 2, 13, 10, 4, 10, 14, 5, 5, 10, 5, 25, 16, 3, 12, 1, 13, 12, 15, 10, 10, 10, 16, 14, 16, 2, 11, 14, 39, 2, 1, 10, 10, 11, 10, 25, 14, 2, 15, 1, 5, 39, 1, 16, 12, 38, 6, 9, 37, 5, 5, 8, 37, 12, 5, 9, 35, 6, 5, 5, 23, 9, 8, 8, 13, 35, 13, 8, 5, 23, 13, 5, 1, 8, 23, 1, 12, 12, 5, 12, 5, 12, 5, 8, 8, 12, 5, 8, 12, 12, 12, 6, 12, 8, 12, 13, 6, 6, 36, 12, 19, 12, 23, 12, 12, 12, 42, 35, 13, 35, 17, 13, 23, 35, 13, 10, 22, 33, 13, 26, 20, 30, 33, 18, 24, 18, 14, 28, 40, 27, 21, 28, 34, 32, 31, 29, 38, 46, 50, 43, 45, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 23, 43, 43, 43, 43, 23, 43, 43, 43, 43, 23, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 43, 44, 44, 44, 48, 48, 48, 48, 48, 48, 48, 48, 48, 48, 48, 48, 47, 48, 48, 48, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 50, 50, 41, 
};

int atune_saxpy(saxpy_arguments)
{
   int bestkernel = besttable_saxpy[n_idx];
   switch (bestkernel)
   {
   case 1:
      saxpy_v1<1, 16, 1, 1>(alpha, x_d, y_d, n);
      break;
   case 2:
      saxpy_v1<1, 16, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 3:
      saxpy_v1<1, 16, 1, 8>(alpha, x_d, y_d, n);
      break;
   case 4:
      saxpy_v1<1, 16, 1, 16>(alpha, x_d, y_d, n);
      break;
   case 5:
      saxpy_v1<1, 32, 1, 1>(alpha, x_d, y_d, n);
      break;
   case 6:
      saxpy_v1<1, 32, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 7:
      saxpy_v1<1, 32, 1, 16>(alpha, x_d, y_d, n);
      break;
   case 8:
      saxpy_v1<1, 64, 1, 1>(alpha, x_d, y_d, n);
      break;
   case 9:
      saxpy_v1<1, 64, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 10:
      saxpy_v1<1, 64, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 11:
      saxpy_v1<1, 64, 1, 16>(alpha, x_d, y_d, n);
      break;
   case 12:
      saxpy_v1<1, 128, 1, 1>(alpha, x_d, y_d, n);
      break;
   case 13:
      saxpy_v1<1, 128, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 14:
      saxpy_v1<1, 128, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 15:
      saxpy_v1<1, 128, 1, 8>(alpha, x_d, y_d, n);
      break;
   case 16:
      saxpy_v1<1, 128, 1, 16>(alpha, x_d, y_d, n);
      break;
   case 17:
      saxpy_v1<1, 160, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 18:
      saxpy_v1<1, 160, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 19:
      saxpy_v1<1, 192, 1, 1>(alpha, x_d, y_d, n);
      break;
   case 20:
      saxpy_v1<1, 192, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 21:
      saxpy_v1<1, 192, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 22:
      saxpy_v1<1, 224, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 23:
      saxpy_v1<1, 256, 1, 1>(alpha, x_d, y_d, n);
      break;
   case 24:
      saxpy_v1<1, 256, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 25:
      saxpy_v1<1, 256, 1, 16>(alpha, x_d, y_d, n);
      break;
   case 26:
      saxpy_v1<1, 288, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 27:
      saxpy_v1<1, 288, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 28:
      saxpy_v1<1, 320, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 29:
      saxpy_v1<1, 320, 1, 8>(alpha, x_d, y_d, n);
      break;
   case 30:
      saxpy_v1<1, 416, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 31:
      saxpy_v1<1, 416, 1, 8>(alpha, x_d, y_d, n);
      break;
   case 32:
      saxpy_v1<1, 448, 1, 8>(alpha, x_d, y_d, n);
      break;
   case 33:
      saxpy_v1<1, 480, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 34:
      saxpy_v1<1, 480, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 35:
      saxpy_v1<1, 512, 1, 1>(alpha, x_d, y_d, n);
      break;
   case 36:
      saxpy_v1<1, 512, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 37:
      saxpy_v1<1, 512, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 38:
      saxpy_v1<1, 512, 1, 8>(alpha, x_d, y_d, n);
      break;
   case 39:
      saxpy_v1<1, 512, 1, 16>(alpha, x_d, y_d, n);
      break;
   case 40:
      saxpy_v1<1, 1024, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 41:
      saxpy_v1<1, 1024, 1, 8>(alpha, x_d, y_d, n);
      break;
   case 42:
      saxpy_v2<1, 64, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 43:
      saxpy_v2<1, 128, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 44:
      saxpy_v2<1, 160, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 45:
      saxpy_v2<1, 224, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 46:
      saxpy_v2<1, 256, 1, 8>(alpha, x_d, y_d, n);
      break;
   case 47:
      saxpy_v2<1, 480, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 48:
      saxpy_v2<1, 512, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 49:
      saxpy_v2<1, 1024, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 50:
      saxpy_v2<1, 1024, 1, 8>(alpha, x_d, y_d, n);
      break;
   default:
      printf("\nsaxpy.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
