//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 316227767
#define m_grid 1
#define n_grid 272
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_scopy[] = { 
8, 20, 8, 38, 38, 38, 8, 38, 8, 8, 38, 38, 8, 33, 33, 13, 5, 9, 7, 23, 13, 15, 12, 2, 8, 11, 4, 4, 13, 5, 5, 23, 17, 14, 14, 10, 16, 11, 17, 27, 17, 12, 17, 14, 26, 24, 27, 23, 14, 8, 35, 9, 17, 11, 17, 14, 17, 10, 3, 14, 17, 17, 31, 25, 8, 14, 14, 13, 3, 8, 13, 2, 24, 13, 13, 8, 13, 8, 8, 13, 29, 2, 14, 8, 30, 13, 38, 40, 6, 8, 13, 1, 8, 13, 33, 13, 13, 8, 2, 13, 13, 22, 13, 8, 32, 37, 8, 8, 8, 22, 8, 8, 8, 9, 13, 8, 8, 8, 8, 8, 8, 23, 3, 33, 13, 13, 13, 13, 33, 9, 39, 14, 38, 23, 14, 22, 39, 22, 39, 14, 23, 14, 23, 18, 23, 34, 43, 47, 24, 44, 24, 36, 19, 28, 19, 21, 28, 24, 48, 50, 46, 46, 45, 45, 45, 45, 45, 49, 45, 45, 49, 49, 49, 45, 45, 45, 49, 45, 45, 45, 45, 45, 49, 45, 45, 49, 42, 45, 49, 45, 45, 45, 49, 45, 45, 49, 45, 45, 41, 45, 42, 45, 49, 42, 51, 42, 41, 45, 42, 49, 51, 51, 51, 42, 41, 51, 41, 51, 51, 49, 41, 51, 51, 51, 41, 41, 41, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 51, 52, 52, 52, 
};

int atune_scopy(scopy_arguments)
{
   int bestkernel = besttable_scopy[n_idx];
   switch (bestkernel)
   {
   case 1:
      scopy_v1<1, 16, 1, 1>(x_d, y_d, n);
      break;
   case 2:
      scopy_v1<1, 16, 1, 2>(x_d, y_d, n);
      break;
   case 3:
      scopy_v1<1, 16, 1, 4>(x_d, y_d, n);
      break;
   case 4:
      scopy_v1<1, 16, 1, 8>(x_d, y_d, n);
      break;
   case 5:
      scopy_v1<1, 16, 1, 16>(x_d, y_d, n);
      break;
   case 6:
      scopy_v1<1, 32, 1, 1>(x_d, y_d, n);
      break;
   case 7:
      scopy_v1<1, 32, 1, 8>(x_d, y_d, n);
      break;
   case 8:
      scopy_v1<1, 64, 1, 1>(x_d, y_d, n);
      break;
   case 9:
      scopy_v1<1, 64, 1, 2>(x_d, y_d, n);
      break;
   case 10:
      scopy_v1<1, 64, 1, 4>(x_d, y_d, n);
      break;
   case 11:
      scopy_v1<1, 64, 1, 8>(x_d, y_d, n);
      break;
   case 12:
      scopy_v1<1, 64, 1, 16>(x_d, y_d, n);
      break;
   case 13:
      scopy_v1<1, 128, 1, 1>(x_d, y_d, n);
      break;
   case 14:
      scopy_v1<1, 128, 1, 2>(x_d, y_d, n);
      break;
   case 15:
      scopy_v1<1, 128, 1, 4>(x_d, y_d, n);
      break;
   case 16:
      scopy_v1<1, 128, 1, 8>(x_d, y_d, n);
      break;
   case 17:
      scopy_v1<1, 128, 1, 16>(x_d, y_d, n);
      break;
   case 18:
      scopy_v1<1, 160, 1, 2>(x_d, y_d, n);
      break;
   case 19:
      scopy_v1<1, 160, 1, 4>(x_d, y_d, n);
      break;
   case 20:
      scopy_v1<1, 192, 1, 8>(x_d, y_d, n);
      break;
   case 21:
      scopy_v1<1, 224, 1, 4>(x_d, y_d, n);
      break;
   case 22:
      scopy_v1<1, 256, 1, 1>(x_d, y_d, n);
      break;
   case 23:
      scopy_v1<1, 256, 1, 2>(x_d, y_d, n);
      break;
   case 24:
      scopy_v1<1, 256, 1, 4>(x_d, y_d, n);
      break;
   case 25:
      scopy_v1<1, 256, 1, 8>(x_d, y_d, n);
      break;
   case 26:
      scopy_v1<1, 256, 1, 16>(x_d, y_d, n);
      break;
   case 27:
      scopy_v1<1, 288, 1, 1>(x_d, y_d, n);
      break;
   case 28:
      scopy_v1<1, 288, 1, 4>(x_d, y_d, n);
      break;
   case 29:
      scopy_v1<1, 320, 1, 1>(x_d, y_d, n);
      break;
   case 30:
      scopy_v1<1, 384, 1, 1>(x_d, y_d, n);
      break;
   case 31:
      scopy_v1<1, 416, 1, 1>(x_d, y_d, n);
      break;
   case 32:
      scopy_v1<1, 448, 1, 2>(x_d, y_d, n);
      break;
   case 33:
      scopy_v1<1, 512, 1, 1>(x_d, y_d, n);
      break;
   case 34:
      scopy_v1<1, 512, 1, 2>(x_d, y_d, n);
      break;
   case 35:
      scopy_v1<1, 512, 1, 8>(x_d, y_d, n);
      break;
   case 36:
      scopy_v1<1, 1024, 1, 4>(x_d, y_d, n);
      break;
   case 37:
      scopy_v2<1, 16, 1, 2>(x_d, y_d, n);
      break;
   case 38:
      scopy_v2<1, 64, 1, 2>(x_d, y_d, n);
      break;
   case 39:
      scopy_v2<1, 128, 1, 2>(x_d, y_d, n);
      break;
   case 40:
      scopy_v2<1, 128, 1, 4>(x_d, y_d, n);
      break;
   case 41:
      scopy_v2<1, 160, 1, 2>(x_d, y_d, n);
      break;
   case 42:
      scopy_v2<1, 192, 1, 2>(x_d, y_d, n);
      break;
   case 43:
      scopy_v2<1, 192, 1, 4>(x_d, y_d, n);
      break;
   case 44:
      scopy_v2<1, 224, 1, 4>(x_d, y_d, n);
      break;
   case 45:
      scopy_v2<1, 256, 1, 2>(x_d, y_d, n);
      break;
   case 46:
      scopy_v2<1, 352, 1, 8>(x_d, y_d, n);
      break;
   case 47:
      scopy_v2<1, 416, 1, 4>(x_d, y_d, n);
      break;
   case 48:
      scopy_v2<1, 448, 1, 8>(x_d, y_d, n);
      break;
   case 49:
      scopy_v2<1, 512, 1, 2>(x_d, y_d, n);
      break;
   case 50:
      scopy_v2<1, 512, 1, 8>(x_d, y_d, n);
      break;
   case 51:
      scopy_v2<1, 1024, 1, 4>(x_d, y_d, n);
      break;
   case 52:
      scopy_v2<1, 1024, 1, 8>(x_d, y_d, n);
      break;
   default:
      printf("\nscopy.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
