//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 316227767
#define m_grid 1
#define n_grid 68
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_sdot[] = { 
15, 15, 16, 15, 16, 16, 16, 16, 27, 4, 8, 4, 11, 5, 3, 7, 1, 1, 1, 1, 6, 2, 6, 2, 3, 25, 9, 7, 19, 10, 26, 26, 22, 20, 20, 28, 17, 18, 18, 23, 24, 21, 21, 14, 17, 11, 11, 12, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 12, 12, 13, 13, 13, 30, 30, 29, 
};

int atune_sdot(sdot_arguments)
{
   int bestkernel = besttable_sdot[n_idx];
   switch (bestkernel)
   {
   case 1:
      sdot_v1<1, 16, 1, 1>(x_d, y_d, z_d, n);
      break;
   case 2:
      sdot_v1<1, 16, 1, 2>(x_d, y_d, z_d, n);
      break;
   case 3:
      sdot_v1<1, 16, 1, 4>(x_d, y_d, z_d, n);
      break;
   case 4:
      sdot_v1<1, 16, 1, 64>(x_d, y_d, z_d, n);
      break;
   case 5:
      sdot_v1<1, 16, 1, 256>(x_d, y_d, z_d, n);
      break;
   case 6:
      sdot_v1<1, 32, 1, 1>(x_d, y_d, z_d, n);
      break;
   case 7:
      sdot_v1<1, 32, 1, 2>(x_d, y_d, z_d, n);
      break;
   case 8:
      sdot_v1<1, 32, 1, 8>(x_d, y_d, z_d, n);
      break;
   case 9:
      sdot_v1<1, 64, 1, 2>(x_d, y_d, z_d, n);
      break;
   case 10:
      sdot_v1<1, 64, 1, 4>(x_d, y_d, z_d, n);
      break;
   case 11:
      sdot_v1<1, 64, 1, 8>(x_d, y_d, z_d, n);
      break;
   case 12:
      sdot_v1<1, 64, 1, 16>(x_d, y_d, z_d, n);
      break;
   case 13:
      sdot_v1<1, 64, 1, 32>(x_d, y_d, z_d, n);
      break;
   case 14:
      sdot_v1<1, 64, 1, 64>(x_d, y_d, z_d, n);
      break;
   case 15:
      sdot_v1<1, 96, 1, 1>(x_d, y_d, z_d, n);
      break;
   case 16:
      sdot_v1<1, 96, 1, 2>(x_d, y_d, z_d, n);
      break;
   case 17:
      sdot_v1<1, 96, 1, 8>(x_d, y_d, z_d, n);
      break;
   case 18:
      sdot_v1<1, 96, 1, 16>(x_d, y_d, z_d, n);
      break;
   case 19:
      sdot_v1<1, 128, 1, 1>(x_d, y_d, z_d, n);
      break;
   case 20:
      sdot_v1<1, 128, 1, 4>(x_d, y_d, z_d, n);
      break;
   case 21:
      sdot_v1<1, 128, 1, 16>(x_d, y_d, z_d, n);
      break;
   case 22:
      sdot_v1<1, 192, 1, 4>(x_d, y_d, z_d, n);
      break;
   case 23:
      sdot_v1<1, 192, 1, 8>(x_d, y_d, z_d, n);
      break;
   case 24:
      sdot_v1<1, 224, 1, 8>(x_d, y_d, z_d, n);
      break;
   case 25:
      sdot_v1<1, 256, 1, 1>(x_d, y_d, z_d, n);
      break;
   case 26:
      sdot_v1<1, 256, 1, 2>(x_d, y_d, z_d, n);
      break;
   case 27:
      sdot_v1<1, 256, 1, 64>(x_d, y_d, z_d, n);
      break;
   case 28:
      sdot_v1<1, 320, 1, 4>(x_d, y_d, z_d, n);
      break;
   case 29:
      sdot_v1<1, 320, 1, 16>(x_d, y_d, z_d, n);
      break;
   case 30:
      sdot_v1<1, 480, 1, 8>(x_d, y_d, z_d, n);
      break;
   default:
      printf("\nsdot.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
