//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 316227767
#define m_grid 1
#define n_grid 272
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_smset[] = { 
1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 17, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 17, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 18, 1, 1, 1, 1, 1, 1, 9, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 9, 1, 1, 1, 1, 1, 9, 1, 1, 1, 1, 1, 1, 1, 1, 27, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 17, 23, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 28, 1, 1, 5, 5, 12, 5, 10, 6, 13, 6, 22, 6, 20, 6, 19, 6, 13, 8, 19, 15, 25, 14, 6, 11, 6, 6, 16, 6, 18, 6, 10, 10, 10, 5, 5, 5, 6, 4, 4, 9, 4, 5, 5, 9, 4, 5, 5, 4, 4, 5, 4, 5, 5, 4, 5, 4, 4, 5, 4, 5, 4, 4, 9, 4, 4, 5, 9, 9, 5, 9, 5, 4, 9, 5, 5, 5, 5, 5, 5, 5, 9, 9, 4, 5, 7, 4, 9, 9, 5, 9, 5, 4, 9, 5, 5, 24, 24, 21, 24, 24, 21, 21, 21, 21, 21, 23, 24, 21, 23, 21, 23, 24, 24, 26, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 30, 30, 30, 
};

int atune_smset(smset_arguments)
{
   int bestkernel = besttable_smset[n_idx];
   switch (bestkernel)
   {
   case 1:
      cudaMemset(y_d, val, sizeof(y_d[0]) * n);
      break;
   case 2:
      smset_v1<1, 32, 1, 1>(val, y_d, n);
      break;
   case 3:
      smset_v1<1, 32, 1, 2>(val, y_d, n);
      break;
   case 4:
      smset_v1<1, 32, 1, 8>(val, y_d, n);
      break;
   case 5:
      smset_v1<1, 64, 1, 4>(val, y_d, n);
      break;
   case 6:
      smset_v1<1, 64, 1, 8>(val, y_d, n);
      break;
   case 7:
      smset_v1<1, 96, 1, 2>(val, y_d, n);
      break;
   case 8:
      smset_v1<1, 96, 1, 8>(val, y_d, n);
      break;
   case 9:
      smset_v1<1, 128, 1, 2>(val, y_d, n);
      break;
   case 10:
      smset_v1<1, 128, 1, 4>(val, y_d, n);
      break;
   case 11:
      smset_v1<1, 128, 1, 8>(val, y_d, n);
      break;
   case 12:
      smset_v1<1, 160, 1, 2>(val, y_d, n);
      break;
   case 13:
      smset_v1<1, 160, 1, 4>(val, y_d, n);
      break;
   case 14:
      smset_v1<1, 192, 1, 8>(val, y_d, n);
      break;
   case 15:
      smset_v1<1, 224, 1, 4>(val, y_d, n);
      break;
   case 16:
      smset_v1<1, 224, 1, 8>(val, y_d, n);
      break;
   case 17:
      smset_v1<1, 256, 1, 1>(val, y_d, n);
      break;
   case 18:
      smset_v1<1, 256, 1, 8>(val, y_d, n);
      break;
   case 19:
      smset_v1<1, 320, 1, 4>(val, y_d, n);
      break;
   case 20:
      smset_v1<1, 352, 1, 4>(val, y_d, n);
      break;
   case 21:
      smset_v1<1, 416, 1, 2>(val, y_d, n);
      break;
   case 22:
      smset_v1<1, 416, 1, 4>(val, y_d, n);
      break;
   case 23:
      smset_v1<1, 448, 1, 2>(val, y_d, n);
      break;
   case 24:
      smset_v1<1, 480, 1, 2>(val, y_d, n);
      break;
   case 25:
      smset_v1<1, 480, 1, 4>(val, y_d, n);
      break;
   case 26:
      smset_v1<1, 512, 1, 2>(val, y_d, n);
      break;
   case 27:
      smset_v1<1, 512, 1, 4>(val, y_d, n);
      break;
   case 28:
      smset_v1<1, 512, 1, 8>(val, y_d, n);
      break;
   case 29:
      smset_v1<1, 1024, 1, 4>(val, y_d, n);
      break;
   case 30:
      smset_v1<1, 1024, 1, 8>(val, y_d, n);
      break;
   default:
      printf("\nsmset.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
