//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 316227767
#define m_grid 1
#define n_grid 68
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_snrm2[] = { 
5, 5, 5, 5, 5, 5, 5, 5, 5, 1, 10, 11, 1, 1, 1, 1, 6, 1, 3, 6, 2, 7, 2, 12, 23, 8, 8, 16, 21, 9, 4, 13, 18, 17, 24, 20, 25, 19, 19, 19, 19, 19, 20, 28, 27, 22, 15, 19, 19, 19, 19, 19, 14, 19, 19, 19, 19, 19, 19, 19, 19, 19, 19, 19, 14, 15, 15, 26, 
};

int atune_snrm2(snrm2_arguments)
{
   int bestkernel = besttable_snrm2[n_idx];
   switch (bestkernel)
   {
   case 1:
      snrm2_v1<1, 16, 1, 1>(x_d, y_d, n);
      break;
   case 2:
      snrm2_v1<1, 16, 1, 2>(x_d, y_d, n);
      break;
   case 3:
      snrm2_v1<1, 16, 1, 4>(x_d, y_d, n);
      break;
   case 4:
      snrm2_v1<1, 16, 1, 8>(x_d, y_d, n);
      break;
   case 5:
      snrm2_v1<1, 16, 1, 256>(x_d, y_d, n);
      break;
   case 6:
      snrm2_v1<1, 32, 1, 1>(x_d, y_d, n);
      break;
   case 7:
      snrm2_v1<1, 32, 1, 2>(x_d, y_d, n);
      break;
   case 8:
      snrm2_v1<1, 32, 1, 4>(x_d, y_d, n);
      break;
   case 9:
      snrm2_v1<1, 32, 1, 8>(x_d, y_d, n);
      break;
   case 10:
      snrm2_v1<1, 32, 1, 32>(x_d, y_d, n);
      break;
   case 11:
      snrm2_v1<1, 32, 1, 256>(x_d, y_d, n);
      break;
   case 12:
      snrm2_v1<1, 64, 1, 1>(x_d, y_d, n);
      break;
   case 13:
      snrm2_v1<1, 64, 1, 4>(x_d, y_d, n);
      break;
   case 14:
      snrm2_v1<1, 64, 1, 32>(x_d, y_d, n);
      break;
   case 15:
      snrm2_v1<1, 64, 1, 64>(x_d, y_d, n);
      break;
   case 16:
      snrm2_v1<1, 96, 1, 2>(x_d, y_d, n);
      break;
   case 17:
      snrm2_v1<1, 96, 1, 4>(x_d, y_d, n);
      break;
   case 18:
      snrm2_v1<1, 96, 1, 8>(x_d, y_d, n);
      break;
   case 19:
      snrm2_v1<1, 96, 1, 16>(x_d, y_d, n);
      break;
   case 20:
      snrm2_v1<1, 128, 1, 16>(x_d, y_d, n);
      break;
   case 21:
      snrm2_v1<1, 160, 1, 1>(x_d, y_d, n);
      break;
   case 22:
      snrm2_v1<1, 160, 1, 32>(x_d, y_d, n);
      break;
   case 23:
      snrm2_v1<1, 256, 1, 2>(x_d, y_d, n);
      break;
   case 24:
      snrm2_v1<1, 256, 1, 4>(x_d, y_d, n);
      break;
   case 25:
      snrm2_v1<1, 256, 1, 8>(x_d, y_d, n);
      break;
   case 26:
      snrm2_v1<1, 320, 1, 16>(x_d, y_d, n);
      break;
   case 27:
      snrm2_v1<1, 320, 1, 32>(x_d, y_d, n);
      break;
   case 28:
      snrm2_v1<1, 448, 1, 16>(x_d, y_d, n);
      break;
   default:
      printf("\nsnrm2.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
