//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 316227767
#define m_grid 1
#define n_grid 68
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_sread[] = { 
1, 4, 1, 4, 4, 4, 4, 4, 4, 1, 4, 12, 14, 3, 12, 19, 10, 11, 12, 11, 12, 27, 16, 6, 9, 8, 2, 6, 8, 23, 15, 16, 9, 8, 17, 22, 22, 10, 5, 7, 7, 21, 13, 25, 22, 18, 22, 22, 22, 18, 18, 22, 22, 22, 18, 18, 22, 18, 22, 18, 20, 24, 28, 28, 28, 29, 29, 26, 
};

int atune_sread(sread_arguments)
{
   int bestkernel = besttable_sread[n_idx];
   switch (bestkernel)
   {
   case 1:
      sread_v1<1, 16, 1, 1>(x_d, n);
      break;
   case 2:
      sread_v1<1, 32, 1, 2>(x_d, n);
      break;
   case 3:
      sread_v1<1, 256, 1, 1>(x_d, n);
      break;
   case 4:
      sread_v1<1, 320, 1, 16>(x_d, n);
      break;
   case 5:
      sread_v1<1, 352, 1, 8>(x_d, n);
      break;
   case 6:
      sread_v1<1, 416, 1, 1>(x_d, n);
      break;
   case 7:
      sread_v1<1, 416, 1, 8>(x_d, n);
      break;
   case 8:
      sread_v1<1, 512, 1, 1>(x_d, n);
      break;
   case 9:
      sread_v1<1, 512, 1, 2>(x_d, n);
      break;
   case 10:
      sread_v1<1, 512, 1, 4>(x_d, n);
      break;
   case 11:
      sread_v1<1, 512, 1, 8>(x_d, n);
      break;
   case 12:
      sread_v1<1, 512, 1, 16>(x_d, n);
      break;
   case 13:
      sread_v1<1, 1024, 1, 16>(x_d, n);
      break;
   case 14:
      sread_v2<1, 16, 1, 2>(x_d, n);
      break;
   case 15:
      sread_v2<1, 32, 1, 2>(x_d, n);
      break;
   case 16:
      sread_v2<1, 64, 1, 2>(x_d, n);
      break;
   case 17:
      sread_v2<1, 64, 1, 4>(x_d, n);
      break;
   case 18:
      sread_v2<1, 64, 1, 8>(x_d, n);
      break;
   case 19:
      sread_v2<1, 64, 1, 16>(x_d, n);
      break;
   case 20:
      sread_v2<1, 96, 1, 8>(x_d, n);
      break;
   case 21:
      sread_v2<1, 96, 1, 16>(x_d, n);
      break;
   case 22:
      sread_v2<1, 128, 1, 4>(x_d, n);
      break;
   case 23:
      sread_v2<1, 192, 1, 2>(x_d, n);
      break;
   case 24:
      sread_v2<1, 224, 1, 4>(x_d, n);
      break;
   case 25:
      sread_v2<1, 224, 1, 16>(x_d, n);
      break;
   case 26:
      sread_v2<1, 320, 1, 16>(x_d, n);
      break;
   case 27:
      sread_v2<1, 512, 1, 2>(x_d, n);
      break;
   case 28:
      sread_v2<1, 512, 1, 4>(x_d, n);
      break;
   case 29:
      sread_v2<1, 512, 1, 8>(x_d, n);
      break;
   default:
      printf("\nsread.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
