//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 316227767
#define m_grid 1
#define n_grid 272
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_srot[] = { 
35, 53, 54, 34, 34, 54, 37, 26, 37, 39, 9, 40, 26, 10, 26, 4, 18, 11, 37, 57, 16, 15, 1, 16, 16, 5, 18, 11, 56, 57, 20, 18, 56, 38, 7, 3, 33, 22, 3, 32, 8, 38, 40, 11, 25, 17, 15, 38, 5, 33, 14, 14, 33, 25, 16, 35, 26, 56, 23, 20, 33, 55, 20, 36, 24, 26, 38, 40, 2, 32, 20, 20, 38, 32, 5, 5, 5, 21, 57, 1, 5, 5, 5, 32, 5, 32, 52, 20, 21, 32, 32, 32, 32, 20, 32, 12, 12, 21, 6, 12, 5, 5, 32, 20, 20, 20, 52, 52, 12, 12, 5, 20, 6, 52, 32, 32, 20, 52, 32, 12, 20, 20, 12, 32, 20, 32, 32, 52, 21, 32, 28, 13, 27, 32, 21, 32, 33, 30, 48, 45, 51, 14, 42, 22, 30, 43, 31, 31, 46, 22, 44, 21, 46, 19, 19, 49, 19, 19, 50, 32, 32, 21, 53, 32, 48, 53, 59, 32, 42, 59, 32, 32, 21, 59, 21, 32, 32, 32, 32, 32, 28, 52, 41, 32, 28, 32, 32, 52, 52, 28, 29, 29, 52, 27, 32, 20, 47, 29, 32, 28, 47, 29, 52, 52, 47, 47, 52, 52, 29, 21, 27, 41, 47, 27, 27, 52, 41, 27, 27, 27, 27, 27, 27, 27, 41, 47, 47, 41, 41, 41, 47, 41, 52, 47, 52, 47, 52, 52, 52, 52, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 59, 59, 59, 59, 59, 59, 59, 59, 59, 60, 60, 60, 
};

int atune_srot(srot_arguments)
{
   int bestkernel = besttable_srot[n_idx];
   switch (bestkernel)
   {
   case 1:
      srot_v1<1, 16, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 2:
      srot_v1<1, 16, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 3:
      srot_v1<1, 16, 1, 16>(x_d, y_d, c, s, n);
      break;
   case 4:
      srot_v1<1, 16, 1, 32>(x_d, y_d, c, s, n);
      break;
   case 5:
      srot_v1<1, 32, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 6:
      srot_v1<1, 32, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 7:
      srot_v1<1, 32, 1, 8>(x_d, y_d, c, s, n);
      break;
   case 8:
      srot_v1<1, 32, 1, 32>(x_d, y_d, c, s, n);
      break;
   case 9:
      srot_v1<1, 32, 1, 64>(x_d, y_d, c, s, n);
      break;
   case 10:
      srot_v1<1, 32, 1, 128>(x_d, y_d, c, s, n);
      break;
   case 11:
      srot_v1<1, 32, 1, 256>(x_d, y_d, c, s, n);
      break;
   case 12:
      srot_v1<1, 64, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 13:
      srot_v1<1, 64, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 14:
      srot_v1<1, 64, 1, 4>(x_d, y_d, c, s, n);
      break;
   case 15:
      srot_v1<1, 64, 1, 16>(x_d, y_d, c, s, n);
      break;
   case 16:
      srot_v1<1, 64, 1, 32>(x_d, y_d, c, s, n);
      break;
   case 17:
      srot_v1<1, 64, 1, 128>(x_d, y_d, c, s, n);
      break;
   case 18:
      srot_v1<1, 64, 1, 256>(x_d, y_d, c, s, n);
      break;
   case 19:
      srot_v1<1, 96, 1, 8>(x_d, y_d, c, s, n);
      break;
   case 20:
      srot_v1<1, 128, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 21:
      srot_v1<1, 128, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 22:
      srot_v1<1, 128, 1, 4>(x_d, y_d, c, s, n);
      break;
   case 23:
      srot_v1<1, 128, 1, 8>(x_d, y_d, c, s, n);
      break;
   case 24:
      srot_v1<1, 128, 1, 32>(x_d, y_d, c, s, n);
      break;
   case 25:
      srot_v1<1, 128, 1, 64>(x_d, y_d, c, s, n);
      break;
   case 26:
      srot_v1<1, 128, 1, 128>(x_d, y_d, c, s, n);
      break;
   case 27:
      srot_v1<1, 160, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 28:
      srot_v1<1, 192, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 29:
      srot_v1<1, 224, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 30:
      srot_v1<1, 224, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 31:
      srot_v1<1, 224, 1, 4>(x_d, y_d, c, s, n);
      break;
   case 32:
      srot_v1<1, 256, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 33:
      srot_v1<1, 256, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 34:
      srot_v1<1, 256, 1, 4>(x_d, y_d, c, s, n);
      break;
   case 35:
      srot_v1<1, 256, 1, 8>(x_d, y_d, c, s, n);
      break;
   case 36:
      srot_v1<1, 256, 1, 16>(x_d, y_d, c, s, n);
      break;
   case 37:
      srot_v1<1, 256, 1, 32>(x_d, y_d, c, s, n);
      break;
   case 38:
      srot_v1<1, 256, 1, 64>(x_d, y_d, c, s, n);
      break;
   case 39:
      srot_v1<1, 256, 1, 128>(x_d, y_d, c, s, n);
      break;
   case 40:
      srot_v1<1, 256, 1, 256>(x_d, y_d, c, s, n);
      break;
   case 41:
      srot_v1<1, 288, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 42:
      srot_v1<1, 288, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 43:
      srot_v1<1, 288, 1, 4>(x_d, y_d, c, s, n);
      break;
   case 44:
      srot_v1<1, 320, 1, 4>(x_d, y_d, c, s, n);
      break;
   case 45:
      srot_v1<1, 352, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 46:
      srot_v1<1, 352, 1, 4>(x_d, y_d, c, s, n);
      break;
   case 47:
      srot_v1<1, 384, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 48:
      srot_v1<1, 416, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 49:
      srot_v1<1, 416, 1, 8>(x_d, y_d, c, s, n);
      break;
   case 50:
      srot_v1<1, 448, 1, 8>(x_d, y_d, c, s, n);
      break;
   case 51:
      srot_v1<1, 480, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 52:
      srot_v1<1, 512, 1, 1>(x_d, y_d, c, s, n);
      break;
   case 53:
      srot_v1<1, 512, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 54:
      srot_v1<1, 512, 1, 4>(x_d, y_d, c, s, n);
      break;
   case 55:
      srot_v1<1, 512, 1, 16>(x_d, y_d, c, s, n);
      break;
   case 56:
      srot_v1<1, 512, 1, 32>(x_d, y_d, c, s, n);
      break;
   case 57:
      srot_v1<1, 512, 1, 128>(x_d, y_d, c, s, n);
      break;
   case 58:
      srot_v1<1, 1024, 1, 2>(x_d, y_d, c, s, n);
      break;
   case 59:
      srot_v1<1, 1024, 1, 4>(x_d, y_d, c, s, n);
      break;
   case 60:
      srot_v1<1, 1024, 1, 8>(x_d, y_d, c, s, n);
      break;
   default:
      printf("\nsrot.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
