//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 316227767
#define m_grid 1
#define n_grid 272
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_sscal[] = { 
56, 59, 56, 52, 52, 52, 52, 5, 42, 49, 13, 9, 5, 44, 43, 9, 9, 16, 7, 15, 14, 43, 14, 14, 9, 16, 42, 8, 8, 16, 41, 9, 9, 9, 6, 7, 8, 15, 14, 1, 6, 9, 43, 42, 45, 45, 3, 26, 3, 25, 9, 16, 8, 26, 41, 16, 25, 10, 44, 4, 41, 24, 25, 42, 14, 44, 42, 45, 7, 6, 7, 7, 7, 6, 12, 6, 6, 6, 13, 12, 4, 3, 3, 3, 3, 41, 1, 14, 13, 3, 7, 4, 50, 3, 4, 23, 7, 12, 3, 42, 13, 23, 9, 48, 6, 50, 23, 6, 3, 3, 12, 6, 12, 2, 4, 6, 6, 6, 7, 7, 6, 23, 12, 24, 12, 8, 13, 24, 7, 8, 51, 13, 12, 42, 13, 13, 13, 13, 13, 24, 24, 27, 39, 15, 30, 24, 42, 25, 31, 11, 46, 46, 18, 15, 22, 35, 29, 24, 36, 31, 31, 14, 38, 40, 25, 14, 14, 20, 9, 15, 47, 33, 61, 37, 47, 62, 17, 42, 13, 38, 24, 13, 55, 42, 32, 21, 42, 55, 24, 24, 55, 9, 34, 19, 42, 24, 17, 57, 28, 24, 24, 53, 19, 21, 19, 24, 17, 24, 34, 19, 60, 17, 42, 17, 17, 34, 17, 17, 63, 58, 60, 17, 54, 17, 17, 17, 54, 17, 17, 54, 17, 17, 17, 17, 54, 54, 28, 58, 28, 58, 28, 28, 42, 42, 42, 42, 42, 42, 42, 42, 46, 46, 46, 46, 46, 46, 46, 46, 46, 46, 46, 46, 46, 46, 46, 46, 46, 46, 46, 64, 64, 64, 
};

int atune_sscal(sscal_arguments)
{
   int bestkernel = besttable_sscal[n_idx];
   switch (bestkernel)
   {
   case 1:
      sscal_v1<1, 16, 1, 1>(alpha, x_d, n);
      break;
   case 2:
      sscal_v1<1, 16, 1, 4>(alpha, x_d, n);
      break;
   case 3:
      sscal_v1<1, 32, 1, 1>(alpha, x_d, n);
      break;
   case 4:
      sscal_v1<1, 32, 1, 2>(alpha, x_d, n);
      break;
   case 5:
      sscal_v1<1, 32, 1, 16>(alpha, x_d, n);
      break;
   case 6:
      sscal_v1<1, 64, 1, 1>(alpha, x_d, n);
      break;
   case 7:
      sscal_v1<1, 64, 1, 2>(alpha, x_d, n);
      break;
   case 8:
      sscal_v1<1, 64, 1, 4>(alpha, x_d, n);
      break;
   case 9:
      sscal_v1<1, 64, 1, 8>(alpha, x_d, n);
      break;
   case 10:
      sscal_v1<1, 64, 1, 16>(alpha, x_d, n);
      break;
   case 11:
      sscal_v1<1, 96, 1, 8>(alpha, x_d, n);
      break;
   case 12:
      sscal_v1<1, 128, 1, 1>(alpha, x_d, n);
      break;
   case 13:
      sscal_v1<1, 128, 1, 2>(alpha, x_d, n);
      break;
   case 14:
      sscal_v1<1, 128, 1, 4>(alpha, x_d, n);
      break;
   case 15:
      sscal_v1<1, 128, 1, 8>(alpha, x_d, n);
      break;
   case 16:
      sscal_v1<1, 128, 1, 16>(alpha, x_d, n);
      break;
   case 17:
      sscal_v1<1, 160, 1, 2>(alpha, x_d, n);
      break;
   case 18:
      sscal_v1<1, 160, 1, 4>(alpha, x_d, n);
      break;
   case 19:
      sscal_v1<1, 192, 1, 2>(alpha, x_d, n);
      break;
   case 20:
      sscal_v1<1, 192, 1, 4>(alpha, x_d, n);
      break;
   case 21:
      sscal_v1<1, 224, 1, 2>(alpha, x_d, n);
      break;
   case 22:
      sscal_v1<1, 224, 1, 16>(alpha, x_d, n);
      break;
   case 23:
      sscal_v1<1, 256, 1, 1>(alpha, x_d, n);
      break;
   case 24:
      sscal_v1<1, 256, 1, 2>(alpha, x_d, n);
      break;
   case 25:
      sscal_v1<1, 256, 1, 8>(alpha, x_d, n);
      break;
   case 26:
      sscal_v1<1, 256, 1, 16>(alpha, x_d, n);
      break;
   case 27:
      sscal_v1<1, 288, 1, 1>(alpha, x_d, n);
      break;
   case 28:
      sscal_v1<1, 288, 1, 2>(alpha, x_d, n);
      break;
   case 29:
      sscal_v1<1, 288, 1, 16>(alpha, x_d, n);
      break;
   case 30:
      sscal_v1<1, 320, 1, 2>(alpha, x_d, n);
      break;
   case 31:
      sscal_v1<1, 320, 1, 8>(alpha, x_d, n);
      break;
   case 32:
      sscal_v1<1, 352, 1, 2>(alpha, x_d, n);
      break;
   case 33:
      sscal_v1<1, 352, 1, 16>(alpha, x_d, n);
      break;
   case 34:
      sscal_v1<1, 384, 1, 2>(alpha, x_d, n);
      break;
   case 35:
      sscal_v1<1, 416, 1, 2>(alpha, x_d, n);
      break;
   case 36:
      sscal_v1<1, 416, 1, 8>(alpha, x_d, n);
      break;
   case 37:
      sscal_v1<1, 416, 1, 16>(alpha, x_d, n);
      break;
   case 38:
      sscal_v1<1, 448, 1, 2>(alpha, x_d, n);
      break;
   case 39:
      sscal_v1<1, 480, 1, 2>(alpha, x_d, n);
      break;
   case 40:
      sscal_v1<1, 480, 1, 4>(alpha, x_d, n);
      break;
   case 41:
      sscal_v1<1, 512, 1, 1>(alpha, x_d, n);
      break;
   case 42:
      sscal_v1<1, 512, 1, 2>(alpha, x_d, n);
      break;
   case 43:
      sscal_v1<1, 512, 1, 4>(alpha, x_d, n);
      break;
   case 44:
      sscal_v1<1, 512, 1, 8>(alpha, x_d, n);
      break;
   case 45:
      sscal_v1<1, 512, 1, 16>(alpha, x_d, n);
      break;
   case 46:
      sscal_v1<1, 1024, 1, 4>(alpha, x_d, n);
      break;
   case 47:
      sscal_v1<1, 1024, 1, 8>(alpha, x_d, n);
      break;
   case 48:
      sscal_v2<1, 16, 1, 2>(alpha, x_d, n);
      break;
   case 49:
      sscal_v2<1, 16, 1, 4>(alpha, x_d, n);
      break;
   case 50:
      sscal_v2<1, 32, 1, 2>(alpha, x_d, n);
      break;
   case 51:
      sscal_v2<1, 64, 1, 2>(alpha, x_d, n);
      break;
   case 52:
      sscal_v2<1, 64, 1, 4>(alpha, x_d, n);
      break;
   case 53:
      sscal_v2<1, 64, 1, 8>(alpha, x_d, n);
      break;
   case 54:
      sscal_v2<1, 96, 1, 4>(alpha, x_d, n);
      break;
   case 55:
      sscal_v2<1, 128, 1, 4>(alpha, x_d, n);
      break;
   case 56:
      sscal_v2<1, 128, 1, 16>(alpha, x_d, n);
      break;
   case 57:
      sscal_v2<1, 192, 1, 2>(alpha, x_d, n);
      break;
   case 58:
      sscal_v2<1, 256, 1, 2>(alpha, x_d, n);
      break;
   case 59:
      sscal_v2<1, 256, 1, 4>(alpha, x_d, n);
      break;
   case 60:
      sscal_v2<1, 288, 1, 2>(alpha, x_d, n);
      break;
   case 61:
      sscal_v2<1, 288, 1, 16>(alpha, x_d, n);
      break;
   case 62:
      sscal_v2<1, 448, 1, 4>(alpha, x_d, n);
      break;
   case 63:
      sscal_v2<1, 512, 1, 2>(alpha, x_d, n);
      break;
   case 64:
      sscal_v2<1, 1024, 1, 8>(alpha, x_d, n);
      break;
   default:
      printf("\nsscal.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
