//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 316227767
#define m_grid 1
#define n_grid 272
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_sswap[] = { 
16, 16, 16, 1, 36, 16, 16, 16, 16, 16, 16, 28, 50, 28, 46, 14, 25, 25, 46, 46, 43, 16, 17, 42, 42, 6, 2, 45, 46, 3, 3, 9, 4, 46, 27, 11, 44, 46, 27, 2, 7, 44, 8, 9, 5, 46, 9, 40, 25, 2, 10, 45, 46, 14, 45, 45, 42, 14, 5, 45, 46, 14, 28, 44, 37, 29, 32, 44, 2, 5, 45, 5, 45, 44, 29, 29, 29, 2, 44, 5, 15, 49, 44, 5, 44, 30, 49, 5, 42, 5, 14, 5, 14, 14, 2, 51, 5, 26, 30, 2, 33, 14, 23, 2, 5, 5, 15, 49, 53, 5, 35, 5, 9, 42, 42, 51, 51, 49, 51, 9, 49, 43, 49, 25, 57, 25, 51, 43, 51, 51, 51, 38, 51, 47, 33, 15, 41, 12, 23, 54, 59, 33, 24, 23, 33, 20, 39, 39, 58, 39, 44, 44, 55, 31, 34, 19, 21, 13, 53, 57, 56, 57, 53, 25, 25, 53, 48, 53, 25, 25, 25, 42, 25, 25, 25, 25, 25, 22, 52, 25, 25, 52, 52, 52, 25, 52, 25, 25, 52, 25, 52, 52, 52, 42, 52, 52, 52, 52, 52, 52, 18, 52, 18, 18, 42, 42, 42, 18, 36, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 42, 42, 30, 30, 30, 30, 36, 30, 36, 42, 36, 36, 42, 42, 42, 42, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 48, 48, 48, 48, 48, 48, 48, 48, 48, 61, 61, 61, 
};

int atune_sswap(sswap_arguments)
{
   int bestkernel = besttable_sswap[n_idx];
   switch (bestkernel)
   {
   case 1:
      cublasSswap(n, x_d, 1, y_d, 1);
      break;
   case 2:
      sswap_v1<1, 16, 1, 1>(x_d, y_d, n);
      break;
   case 3:
      sswap_v1<1, 16, 1, 4>(x_d, y_d, n);
      break;
   case 4:
      sswap_v1<1, 16, 1, 8>(x_d, y_d, n);
      break;
   case 5:
      sswap_v1<1, 32, 1, 1>(x_d, y_d, n);
      break;
   case 6:
      sswap_v1<1, 32, 1, 4>(x_d, y_d, n);
      break;
   case 7:
      sswap_v1<1, 32, 1, 8>(x_d, y_d, n);
      break;
   case 8:
      sswap_v1<1, 32, 1, 16>(x_d, y_d, n);
      break;
   case 9:
      sswap_v1<1, 64, 1, 1>(x_d, y_d, n);
      break;
   case 10:
      sswap_v1<1, 64, 1, 4>(x_d, y_d, n);
      break;
   case 11:
      sswap_v1<1, 64, 1, 16>(x_d, y_d, n);
      break;
   case 12:
      sswap_v1<1, 96, 1, 2>(x_d, y_d, n);
      break;
   case 13:
      sswap_v1<1, 96, 1, 8>(x_d, y_d, n);
      break;
   case 14:
      sswap_v1<1, 128, 1, 1>(x_d, y_d, n);
      break;
   case 15:
      sswap_v1<1, 128, 1, 2>(x_d, y_d, n);
      break;
   case 16:
      sswap_v1<1, 128, 1, 8>(x_d, y_d, n);
      break;
   case 17:
      sswap_v1<1, 128, 1, 16>(x_d, y_d, n);
      break;
   case 18:
      sswap_v1<1, 160, 1, 1>(x_d, y_d, n);
      break;
   case 19:
      sswap_v1<1, 160, 1, 2>(x_d, y_d, n);
      break;
   case 20:
      sswap_v1<1, 160, 1, 4>(x_d, y_d, n);
      break;
   case 21:
      sswap_v1<1, 160, 1, 8>(x_d, y_d, n);
      break;
   case 22:
      sswap_v1<1, 192, 1, 1>(x_d, y_d, n);
      break;
   case 23:
      sswap_v1<1, 192, 1, 2>(x_d, y_d, n);
      break;
   case 24:
      sswap_v1<1, 224, 1, 4>(x_d, y_d, n);
      break;
   case 25:
      sswap_v1<1, 256, 1, 1>(x_d, y_d, n);
      break;
   case 26:
      sswap_v1<1, 256, 1, 2>(x_d, y_d, n);
      break;
   case 27:
      sswap_v1<1, 256, 1, 4>(x_d, y_d, n);
      break;
   case 28:
      sswap_v1<1, 256, 1, 8>(x_d, y_d, n);
      break;
   case 29:
      sswap_v1<1, 256, 1, 16>(x_d, y_d, n);
      break;
   case 30:
      sswap_v1<1, 288, 1, 1>(x_d, y_d, n);
      break;
   case 31:
      sswap_v1<1, 288, 1, 4>(x_d, y_d, n);
      break;
   case 32:
      sswap_v1<1, 288, 1, 8>(x_d, y_d, n);
      break;
   case 33:
      sswap_v1<1, 320, 1, 2>(x_d, y_d, n);
      break;
   case 34:
      sswap_v1<1, 320, 1, 4>(x_d, y_d, n);
      break;
   case 35:
      sswap_v1<1, 352, 1, 1>(x_d, y_d, n);
      break;
   case 36:
      sswap_v1<1, 384, 1, 1>(x_d, y_d, n);
      break;
   case 37:
      sswap_v1<1, 384, 1, 2>(x_d, y_d, n);
      break;
   case 38:
      sswap_v1<1, 448, 1, 2>(x_d, y_d, n);
      break;
   case 39:
      sswap_v1<1, 448, 1, 4>(x_d, y_d, n);
      break;
   case 40:
      sswap_v1<1, 448, 1, 16>(x_d, y_d, n);
      break;
   case 41:
      sswap_v1<1, 480, 1, 1>(x_d, y_d, n);
      break;
   case 42:
      sswap_v1<1, 512, 1, 1>(x_d, y_d, n);
      break;
   case 43:
      sswap_v1<1, 512, 1, 2>(x_d, y_d, n);
      break;
   case 44:
      sswap_v1<1, 512, 1, 4>(x_d, y_d, n);
      break;
   case 45:
      sswap_v1<1, 512, 1, 8>(x_d, y_d, n);
      break;
   case 46:
      sswap_v1<1, 512, 1, 16>(x_d, y_d, n);
      break;
   case 47:
      sswap_v1<1, 1024, 1, 2>(x_d, y_d, n);
      break;
   case 48:
      sswap_v1<1, 1024, 1, 4>(x_d, y_d, n);
      break;
   case 49:
      sswap_v2<1, 32, 1, 2>(x_d, y_d, n);
      break;
   case 50:
      sswap_v2<1, 32, 1, 4>(x_d, y_d, n);
      break;
   case 51:
      sswap_v2<1, 64, 1, 2>(x_d, y_d, n);
      break;
   case 52:
      sswap_v2<1, 96, 1, 2>(x_d, y_d, n);
      break;
   case 53:
      sswap_v2<1, 128, 1, 2>(x_d, y_d, n);
      break;
   case 54:
      sswap_v2<1, 160, 1, 2>(x_d, y_d, n);
      break;
   case 55:
      sswap_v2<1, 160, 1, 4>(x_d, y_d, n);
      break;
   case 56:
      sswap_v2<1, 192, 1, 2>(x_d, y_d, n);
      break;
   case 57:
      sswap_v2<1, 256, 1, 2>(x_d, y_d, n);
      break;
   case 58:
      sswap_v2<1, 416, 1, 4>(x_d, y_d, n);
      break;
   case 59:
      sswap_v2<1, 480, 1, 2>(x_d, y_d, n);
      break;
   case 60:
      sswap_v2<1, 1024, 1, 2>(x_d, y_d, n);
      break;
   case 61:
      sswap_v2<1, 1024, 1, 8>(x_d, y_d, n);
      break;
   default:
      printf("\nsswap.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
