//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 25483
#define n_max 1
#define m_grid 141
#define n_grid 1
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_strsv_col_major[] = { 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
1, 
4, 
6, 
6, 
6, 
6, 
3, 
2, 
5, 
6, 
10, 
10, 
10, 
14, 
12, 
13, 
14, 
11, 
11, 
14, 
16, 
16, 
15, 
15, 
15, 
15, 
15, 
15, 
15, 
15, 
15, 
15, 
17, 
17, 
17, 
17, 
17, 
16, 
16, 
17, 
17, 
18, 
18, 
18, 
18, 
18, 
18, 
18, 
18, 
18, 
7, 
18, 
7, 
8, 
19, 
19, 
19, 
19, 
8, 
19, 
8, 
19, 
8, 
9, 
9, 
9, 
20, 
9, 
9, 
20, 
9, 
9, 
8, 
19, 
19, 
19, 
19, 
19, 
19, 
20, 
20, 
9, 
8, 
19, 
19, 
20, 
9, 
20, 
19, 
19, 
20, 
20, 
20, 
20, 
20, 
20, 
20, 
20, 
20, 
9, 
};

int atune_strsv_col_major(strsv_arguments)
{
   int bestkernel = besttable_strsv_col_major[m_idx * n_grid];
   switch (bestkernel)
   {
   case 1:
      cublasStrsv('L', 'N', 'N', m, A_d, lda, x_d, 1);
      break;
   case 2:
      strsv_col_major_v2<16, 8, 1, 1>(A_d, x_d, work_d, m, lda);
      break;
   case 3:
      strsv_col_major_v2<16, 8, 1, 2>(A_d, x_d, work_d, m, lda);
      break;
   case 4:
      strsv_col_major_v2<16, 8, 1, 4>(A_d, x_d, work_d, m, lda);
      break;
   case 5:
      strsv_col_major_v2<16, 16, 1, 1>(A_d, x_d, work_d, m, lda);
      break;
   case 6:
      strsv_col_major_v2<16, 16, 1, 8>(A_d, x_d, work_d, m, lda);
      break;
   case 7:
      strsv_col_major_v2<32, 4, 4, 4>(A_d, x_d, work_d, m, lda);
      break;
   case 8:
      strsv_col_major_v2<32, 4, 4, 8>(A_d, x_d, work_d, m, lda);
      break;
   case 9:
      strsv_col_major_v2<32, 4, 4, 16>(A_d, x_d, work_d, m, lda);
      break;
   case 10:
      strsv_col_major_v2<32, 8, 1, 1>(A_d, x_d, work_d, m, lda);
      break;
   case 11:
      strsv_col_major_v2<32, 8, 1, 2>(A_d, x_d, work_d, m, lda);
      break;
   case 12:
      strsv_col_major_v2<32, 8, 1, 4>(A_d, x_d, work_d, m, lda);
      break;
   case 13:
      strsv_col_major_v2<32, 8, 1, 8>(A_d, x_d, work_d, m, lda);
      break;
   case 14:
      strsv_col_major_v2<32, 8, 1, 16>(A_d, x_d, work_d, m, lda);
      break;
   case 15:
      strsv_col_major_v2<32, 8, 2, 1>(A_d, x_d, work_d, m, lda);
      break;
   case 16:
      strsv_col_major_v2<32, 8, 2, 2>(A_d, x_d, work_d, m, lda);
      break;
   case 17:
      strsv_col_major_v2<32, 8, 4, 2>(A_d, x_d, work_d, m, lda);
      break;
   case 18:
      strsv_col_major_v2<32, 8, 4, 4>(A_d, x_d, work_d, m, lda);
      break;
   case 19:
      strsv_col_major_v2<32, 8, 4, 8>(A_d, x_d, work_d, m, lda);
      break;
   case 20:
      strsv_col_major_v2<32, 8, 4, 16>(A_d, x_d, work_d, m, lda);
      break;
   default:
      printf("\nstrsv_col_major.h : bestkernelSwitch : problem size not supported (m = %d, bestkernel = %d).\n", m, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
