#pragma once
#include <iostream>
#include <iomanip>
#include "defines.h"

//cudaError mycudaMalloc(void** h_mem ,unsigned long memSize, memoryMode memMode = PAGEABLE);

namespace dense 
{
	enum formats { COL_MAJOR, ROW_MAJOR };
	class matrix {

	private:

	public:
		
		int m;
		int n;
		float *data;
		formats format;

		matrix() { data = NULL; format = COL_MAJOR;};
		
		matrix(const matrix& mat) 
		{
			data = NULL;
			copyset(mat);
		};
		
		matrix(int m, int n) 
		{
			this->m = m;
			this->n = n;
			data = (float *) malloc(m*n*sizeof(float));
			if (data == NULL) { printf("dense::matrix : malloc error."); exit(0); }
		};
		
        matrix& operator=(const matrix& mat)
		{
			copyset(mat);
			return *this;
		}

		void zeroset(int m, int n)
        {
			this->m = m;
			this->n = n;
			data = (float *) malloc(m*n*sizeof(float));
			if (data == NULL) { printf("dense::zeroset : malloc error."); exit(0); }
			for(int i = 0; i < m*n; i++) data[i] = 0;
		}

		void oneset(int m, int n)
        {
			this->m = m;
			this->n = n;
			data = (float *) malloc(m*n*sizeof(float));
			if (data == NULL) { printf("dense::oneset : malloc error."); exit(0); }
			for(int i = 0; i < m*n; i++) data[i] = 1;
		}

		void rowset(int m, int n)
        {
			this->m = m;
			this->n = n;
			data = (float *) malloc(m*n*sizeof(float));
			if (data == NULL) { printf("dense::rowset : malloc error."); exit(0); }
			for(int i = 0; i < n; i++)
				for(int j = 0; j < m; j++) data[i*m+j] = j+1;
		}

		void colset(int m, int n)
        {
			this->m = m;
			this->n = n;
			data = (float *) malloc(m*n*sizeof(float));
			if (data == NULL) { printf("dense::rowset : malloc error."); exit(0); }
			for(int i = 0; i < n; i++)
				for(int j = 0; j < m; j++) data[i*m+j] = i+1;
		}

		void diagset(int m, int n)
        {
			this->m = m;
			this->n = n;
			data = (float *) malloc(m*n*sizeof(float));
			if (data == NULL) { printf("dense::diagset : malloc error."); exit(0); }
			for(int i = 0; i < n; i++)
				for(int j = 0; j < m; j++) data[i*m+j] = j-i+1;
		}
		/*
		void cudarowset(int m, int n)
        {
			this->m = m;
			this->n = n;
			cudaAssert(mycudaMalloc((void**) &data, m*n*sizeof(float), PINNED));
			for(int i = 0; i < n; i++)
				for(int j = 0; j < m; j++) data[i*m+j] = j+1;
		}
		*/
		void randset(int m, int n)
        { 
			this->m = m;
			this->n = n;
			data = (float *) malloc(m*n*sizeof(float));
			if (data == NULL) { printf("dense::randset : malloc error."); exit(0); }
			for(int i = 0; i < m*n; i++) data[i] = rand() / (float)RAND_MAX;
		}
		
		void copyset(const matrix& mat)
        { 
			m = mat.m;
			n = mat.n;
			format = mat.format;
			if (data != NULL) free(data);
			data = (float *) malloc(m*n*sizeof(float));
			if (data == NULL) { printf("dense::copyset : malloc error."); exit(0); }
			for(int i = 0; i < m*n; i++) data[i] = mat.data[i];
		}

		void clone(const matrix& mat)
        { 
			m = mat.m;
			n = mat.n;
			format = mat.format;
			if (data != NULL && data != mat.data) free(data);
			data = mat.data;
		}

		void swap(matrix& mat)
        {
			int mtemp = m;
			int ntemp = n;
			formats formattemp = format;
			float *datatemp = data;
			m = mat.m;			
			n = mat.n;
			format = mat.format;
			data = mat.data;
			mat.m = mtemp;
			mat.n = ntemp;
			mat.format = formattemp;
			mat.data = datatemp;
		}

		void reset(float val)
        {
			for(int i = 0; i < m*n; i++) data[i] = val;
		}

		void transpose(const matrix& mat)
        { 
			m = mat.n;
			n = mat.m;
			if (data != NULL) free(data);
			data = (float *) malloc(m*n*sizeof(float));
			if (data == NULL) { printf("dense::transpose : malloc error."); exit(0); }
			for(int i = 0; i < m; i++)
				for(int j = 0; j < n; j++) data[j*m+i] = mat.data[i*n+j];
		}

		void sub(const matrix &mat)
        { 
			if (m != mat.m || n != mat.n) {
				printf("dense::diff : matrix sizes must be equal.\n");
				exit(EXIT_FAILURE);
			}
			for(int i = 0; i < m*n; i++) data[i] -= mat.data[i];
		}

		void add(const matrix &mat)
        { 
			if (m != mat.m || n != mat.n) {
				printf("dense::diff : matrix sizes must be equal.\n");
				exit(EXIT_FAILURE);
			}
			for(int i = 0; i < m*n; i++) data[i] += mat.data[i];
		}

		float normfro()
        { 
			float sumsquares = 0;
			for(int i = 0; i < m*n; i++) sumsquares += data[i]*data[i];
			return sqrt(sumsquares);
		}

		float normfro(const matrix &mat)
        { 
			matrix D(*this);
			D.sub(mat);
			return D.normfro();
		}

		float max()
        { 
			float maxval = 0;
			for(int i = 0; i < m*n; i++) if (abs(data[i]) > maxval) maxval = abs(data[i]);
			return maxval;
		}

		float max(const matrix &mat)
        { 
			matrix D(*this);
			D.sub(mat);
			return D.max();
		}

		void print()
        {
			print("");
		}
	
#define PRECISION 6
		void print(char name[])
        {
			std::cout << "dense_matrix";
			if (format == ROW_MAJOR)
				std::cout << "< row major >";
			else
				std::cout << "< column major >";
			std::cout << "<" << m << ", " << n << ">\n";
			std::cout << name << " = [";
			std::cout.precision(PRECISION);
			for(int i = 0; i < m; i++)
			{
				std::cout << std::endl;
				if (format == ROW_MAJOR)
					for(int j = 0; j < n; j++) std::cout << " " << std::setw(PRECISION + 4) << data[i*n + j];
				else
					for(int j = 0; j < n; j++) std::cout << " " << std::setw(PRECISION + 4) << data[j*m + i];
			}
			std::cout << std::endl << "];" << std::endl;
		}
	};
	
}


