#include "defines.h"
#include "isamax_kernels.cu"

#if not defined (_TESLA) && not defined (_FERMI)
#error Requires an architechture flag (either _TESLA or _FERMI) to compile.
#endif

#define switchParameters x_d, y_d
#include "level1_templatemacros.h"

#ifdef _ATUNE

// When building atune
#define arguments isamax_parameters
#define call_arguments isamax_call_parameters
#define switchMacro workSizeSwitchPow_n
#define templatedec

#else

// When building library
#define BLOCKSIZE BLOCKSIZE_n
#define blockSize_n BLOCKSIZE_n
#define workSize_m WORKSIZE_m
#define workSize_n WORKSIZE_n
#define arguments isamax_arguments
#define call_arguments isamax_call_arguments
#define switchMacro call_kernel
#define templatedec template<int BLOCKSIZE_m, int BLOCKSIZE_n, int WORKSIZE_m, int WORKSIZE_n>

#endif

templatedec
static int isamax_v1(arguments)
{		
	//
	// Generic version
	//

	// Clear output vector (assume beta=0.0)
	cudaAssert( cudaMemset(y_d, 0, 1 * sizeof(int)) );

	// Determine size of grid
	int gridSize_n = (n + blockSize_n * workSize_n - 1) / (blockSize_n * workSize_n);
	if (gridSize_n > 65535) return -1; 

	// Sizes of the tiled part
	int ntile = (n / (blockSize_n * workSize_n)) * (blockSize_n * workSize_n);
		
	// Kernel specifications
	dim3 dimGrid(gridSize_n);
	dim3 dimBlock(blockSize_n);
		
	// Call nested switch macros for all templated values
	//printf("\nIsamax-%d: n=%d grid_n=%d ntile=%d block_m=%d block_n=%d work_m=%d work_n=%d\n", version, n, gridSize_n, ntile, blockSize_m, blockSize_n, workSize_m, workSize_n);
	switchMacro(isamax_kernel, BLOCKSIZE, 1, workSize_n);
	//workSizeSwitch_mn(isamax_kernel2, x_d, y_d, n, ntile, BLOCKSIZE, workSize_m, workSize_n);
	return 0;
}

#ifdef _ATUNE

// When building atune
template <int BLOCKSIZE_n>
int isamax(isamax_parameters);

template <>
int isamax<BLOCKSIZE>(isamax_parameters)
{
	if (version == 1)
	{
		return isamax_v1(isamax_call_parameters);
	}
	// return "not run" signal to auto-tuner
	return -1;
}

#else

//When building library
#include "atune_headers/isamax.h"
int glas_isamax(int n, float *x_d, int incx, int *y_d) 
{

	if (incx == 1)
	{
		return atune_isamax(isamax_call_arguments);
	}
	else 
	{
		printf("Call to isamax: parameter 3 has an unsupported value (%d)!\n", incx); exit(EXIT_FAILURE);
	}
}

#endif

