#include "defines.h"
#include "scopy_kernels.cu"
#include <cublas.h>

#if not defined (_TESLA) && not defined (_FERMI)
#error Requires an architechture flag (either _TESLA or _FERMI) to compile.
#endif

#define switchParameters x_d, y_d
#include "level1_templatemacros.h"

#ifdef _ATUNE

// When building atune
#define arguments scopy_parameters
#define call_arguments scopy_call_parameters
#define switchMacro workSizeSwitchSmallPow_n
#define templatedec

#else

// When building library
#define BLOCKSIZE BLOCKSIZE_n
#define blockSize_n BLOCKSIZE_n
#define workSize_n WORKSIZE_n
#define arguments scopy_arguments
#define call_arguments scopy_call_arguments
#define switchMacro call_kernel
#define templatedec template<int BLOCKSIZE_m, int BLOCKSIZE_n, int WORKSIZE_m, int WORKSIZE_n>

#endif

templatedec
static int scopy_v1(arguments)
{		
	//
	// Generic version
	//

	// Determine size of grid (make sure that we are within grid dimension limits)
	int gridSize_n = (n + blockSize_n * workSize_n - 1) / (blockSize_n * workSize_n);
	if (gridSize_n > 65535) return -1; 

	// Sizes of the tiled part
	int ntile = (n / (blockSize_n * workSize_n)) * (blockSize_n * workSize_n);
	
	// Kernel specifications
	dim3 dimGrid(gridSize_n);
	dim3 dimBlock(blockSize_n);
	
	// Call nested switch macros for all templated values
	//printf("\nScopy-%d: n=%d grid_n=%d ntile=%d block_m=%d block_n=%d work_m=%d work_n=%d\n", version, n, gridSize_n, ntile, blockSize_m, blockSize_n, workSize_m, workSize_n);
	switchMacro(scopy_kernel1, BLOCKSIZE, 1, workSize_n);
	return 0;
}

templatedec
static int scopy_v2(arguments)
{		
	//
	// Generic version using doubles
	//

	// Determine size of grid
	int gridSize_n = (n + blockSize_n * workSize_n - 1) / (blockSize_n * workSize_n);
	if (gridSize_n > 65535) return -1; 
	
	// Sizes of the tiled part
	int ntile = (n / (blockSize_n * workSize_n)) * (blockSize_n * workSize_n);
	
	// Kernel specifications
	dim3 dimGrid(gridSize_n);
	dim3 dimBlock(blockSize_n);
	
	// Call nested switch macros for all templated values
	//printf("\nScopy-%d: n=%d grid_n=%d ntile=%d block_m=%d block_n=%d work_m=%d work_n=%d\n", version, n, gridSize_n, ntile, blockSize_m, blockSize_n, workSize_m, workSize_n);
	switchMacro(scopy_kernel2, BLOCKSIZE, 1, workSize_n);
	return 0;
}

#ifdef _ATUNE

// When building atune
template <int BLOCKSIZE_n>
int scopy(scopy_parameters);

template <>
int scopy<BLOCKSIZE>(scopy_parameters)
{
	if (version == 1)
	{
		return scopy_v1(scopy_call_parameters);
	}
	else if (version == 2 && !(workSize_n & 1))
	{
		return scopy_v2(scopy_call_parameters);
	}

	// return "not run" signal to auto-tuner
	return -1;
}

#else

//When building library
#include "atune_headers/scopy.h"
int glas_scopy(int n, float *x_d, int incx, float *y_d, int incy) 
{

	if (incx == 1 && incy == 1)
	{
		return atune_scopy(scopy_call_arguments);
	}
	else 
	{
		printf("Call to scopy: parameter 3 or 5 has an unsupported value (%d, %d)!\n", incx, incy); exit(EXIT_FAILURE);
	}
}

#endif
